@@ -10,14 +10,37 @@ GitLab References:
 import os
 import subprocess
 from argparse import Namespace
+from functools import lru_cache
 from pathlib import Path
 from typing import Optional
 
 import requests
-from phylum.ci.ci_base import CIBase
+from connect.utils.terminal.markdown import render
+from phylum.ci.ci_base import CIBase, git_remote
 from phylum.ci.constants import PHYLUM_HEADER
 from phylum.constants import REQ_TIMEOUT
 
+SHA1_ALL_ZEROES = "0000000000000000000000000000000000000000"
+
+
+@lru_cache(maxsize=1)
+def is_in_mr() -> bool:
+    """Indicate if the integration is operating in the context of a merge request pipeline.
+
+    GitLab CI allows for the possibility of running pipelines in different contexts:
+        * On every push, for the last commit in the push (e.g., branch pipelines)
+        * For merge requests (e.g., merge request pipelines)
+
+    Knowing when the context is within a merge request helps to ensure the logic used
+    to determine the lockfile changes is correct. It also helps to ensure output is not
+    attempted to be posted when NOT in the context of a review.
+    """
+    # References:
+    # https://github.com/watson/ci-info/blob/master/vendors.json
+    # https://docs.gitlab.com/ee/ci/pipelines/merge_request_pipelines.html
+    # docs.gitlab.com/ee/ci/variables/predefined_variables.html#predefined-variables-for-merge-request-pipelines
+    return bool(os.getenv("CI_MERGE_REQUEST_ID"))
+
 
 class CIGitLab(CIBase):
     """Provide methods for a GitLab CI environment."""
@@ -25,6 +48,10 @@ class CIGitLab(CIBase):
     def __init__(self, args: Namespace) -> None:
         super().__init__(args)
         self.ci_platform_name = "GitLab CI"
+        if is_in_mr():
+            print(" [-] Pipeline context: merge request pipeline")
+        else:
+            print(" [-] Pipeline context: branch pipeline")
 
     def _check_prerequisites(self) -> None:
         """Ensure the necessary pre-requisites are met and bail when they aren't.
@@ -35,14 +62,17 @@ class CIGitLab(CIBase):
         """
         super()._check_prerequisites()
 
+        # References:
+        # https://github.com/watson/ci-info/blob/master/vendors.json
+        # https://docs.gitlab.com/ee/ci/variables/predefined_variables.html
         if os.getenv("GITLAB_CI") != "true":
             raise SystemExit(" [!] Must be working within the GitLab CI environment")
 
         # A GitLab token with API access is required to use the API (e.g., to post notes/comments).
         # This can be a personal, project, or group access token...and possibly some other types as well.
         # See the GitLab Token Overview Documentation for info: https://docs.gitlab.com/ee/security/token_overview.html
-        gitlab_token = os.getenv("GITLAB_TOKEN")
-        if not gitlab_token:
+        gitlab_token = os.getenv("GITLAB_TOKEN", "")
+        if not gitlab_token and is_in_mr():
             raise SystemExit(" [!] A GitLab token with API access must be set at `GITLAB_TOKEN`")
         self._gitlab_token = gitlab_token
 
@@ -54,9 +84,18 @@ class CIGitLab(CIBase):
     @property
     def phylum_label(self) -> str:
         """Get a custom label for use when submitting jobs with `phylum analyze`."""
-        mr_iid = os.getenv("CI_MERGE_REQUEST_IID", "unknown-IID")
-        mr_title = os.getenv("CI_MERGE_REQUEST_TITLE", "unknown-title")
-        label = f"{self.ci_platform_name}_MR#{mr_iid}_{mr_title}"
+        if is_in_mr():
+            mr_iid = os.getenv("CI_MERGE_REQUEST_IID", "unknown-IID")
+            mr_title = os.getenv("CI_MERGE_REQUEST_TITLE", "unknown-title")
+            label = f"{self.ci_platform_name}_MR#{mr_iid}_{mr_title}"
+        else:
+            current_branch = os.getenv("CI_COMMIT_BRANCH", "unknown-branch")
+            # This is the unique key that git uses to refer to the blob type data object for the lockfile.
+            # Reference: https://git-scm.com/book/en/v2/Git-Internals-Git-Objects
+            cmd = f"git hash-object {self.lockfile}".split()
+            lockfile_hash_object = subprocess.run(cmd, check=True, text=True, capture_output=True).stdout.strip()
+            label = f"{self.ci_platform_name}_{current_branch}_{lockfile_hash_object[:7]}"
+
         label = label.replace(" ", "-")
         return label
 
@@ -64,25 +103,50 @@ class CIGitLab(CIBase):
     def common_lockfile_ancestor_commit(self) -> Optional[str]:
         """Find the common lockfile ancestor commit."""
         # Reference: https://docs.gitlab.com/ee/ci/variables/predefined_variables.html
-        return os.getenv("CI_MERGE_REQUEST_DIFF_BASE_SHA")
+        if is_in_mr():
+            common_ancestor_commit = os.getenv("CI_MERGE_REQUEST_DIFF_BASE_SHA")
+        else:
+            # CI_COMMIT_BEFORE_SHA contains the previous latest commit present on the branch *pipeline*.
+            # It will always be all zeroes in merge request pipelines.
+            # It will be all zeroes for the first pipeline run in branch pipelines,
+            # whether or not the commit the pipeline runs on is the first commit in the branch.
+            # If there are multiple commits between pipeline runs (e.g., push a collection of commits), the value
+            # points to the commit of the previous pipeline run, skipping over all the intermediate commits.
+            # Think of this as a pointer to the previous commit **that ran in the same branch pipeline**.
+            common_ancestor_commit = os.getenv("CI_COMMIT_BEFORE_SHA")
+
+        # There is not an environment variable provided by GitLab CI to show or determine the common ancestor commit
+        # when running in a branch pipeline, on the initial run of that pipeline. Fallback to computing it manually.
+        if common_ancestor_commit == SHA1_ALL_ZEROES:
+            print(" [-] Detected initial branch pipeline run")
+            remote = git_remote()
+            # The default branch is found this way because there is a GitLab runner bug where HEAD is not available:
+            # https://gitlab.com/gitlab-org/gitlab-runner/-/issues/4078
+            default_branch = os.getenv("CI_DEFAULT_BRANCH", "HEAD")
+            # This is a best effort attempt since it is finding the merge base between the current commit
+            # and the default branch instead of finding the exact commit from which the branch was created.
+            cmd = f"git merge-base HEAD refs/remotes/{remote}/{default_branch}".split()
+            try:
+                common_ancestor_commit = subprocess.run(cmd, check=True, capture_output=True, text=True).stdout.strip()
+            except subprocess.CalledProcessError as err:
+                print(f" [!] The common lockfile ancestor commit could not be found: {err}")
+                common_ancestor_commit = None
+
+        return common_ancestor_commit
 
     def _is_lockfile_changed(self, lockfile: Path) -> bool:
         """Predicate for detecting if the given lockfile has changed."""
-        mr_src_branch = os.getenv("CI_MERGE_REQUEST_SOURCE_BRANCH_NAME")
-        mr_tgt_branch = os.getenv("CI_MERGE_REQUEST_TARGET_BRANCH_NAME")
-        mr_diff_base_sha = os.getenv("CI_MERGE_REQUEST_DIFF_BASE_SHA")
-        print(f" [+] CI_MERGE_REQUEST_SOURCE_BRANCH_NAME: {mr_src_branch}")
-        print(f" [+] CI_MERGE_REQUEST_TARGET_BRANCH_NAME: {mr_tgt_branch}")
-        print(f" [+] CI_MERGE_REQUEST_DIFF_BASE_SHA: {mr_diff_base_sha}")
+        diff_base_sha = self.common_lockfile_ancestor_commit
+        print(f" [+] The common lockfile ancestor commit: {diff_base_sha}")
 
         # Assume no change when there isn't enough information to tell
-        if mr_diff_base_sha is None:
+        if diff_base_sha is None:
             return False
 
         try:
             # `--exit-code` will make git exit with 1 if there were differences while 0 means no differences.
             # Any other exit code is an error and a reason to re-raise.
-            cmd = f"git diff --exit-code --quiet {mr_diff_base_sha} -- {lockfile.resolve()}"
+            cmd = f"git diff --exit-code --quiet {diff_base_sha} -- {lockfile.resolve()}"
             subprocess.run(cmd.split(), check=True)
             return False
         except subprocess.CalledProcessError as err:
@@ -92,7 +156,17 @@ class CIGitLab(CIBase):
             raise
 
     def post_output(self) -> None:
-        """Post the output of the analysis as a note (comment) on the GitLab CI Merge Request (MR)."""
+        """Post the output of the analysis.
+
+        Post output directly in the logs regardless of the pipeline context.
+        Post output as a note (comment) on the GitLab CI Merge Request (MR) when operating in a merge request pipeline.
+        """
+        print(f" [+] Analysis output:\n{render(self.analysis_output)}")
+
+        if not is_in_mr():
+            # Can't post the output to the MR when there is no MR
+            return
+
         # API Reference: https://docs.gitlab.com/ee/api/notes.html#merge-requests
         gitlab_api_v4_root_url = os.getenv("CI_API_V4_URL")
         mr_project_id = os.getenv("CI_MERGE_REQUEST_PROJECT_ID")
@@ -111,7 +185,7 @@ class CIGitLab(CIBase):
         if not mr_notes:
             print(" [+] No existing merge request notes found.")
         # NOTE: The API defaults to returning the notes in descending order by the `created_at` field.
-        #       Detecting Phylum notes is done simply by looking for those notes that start with a known string value.
+        #       Detecting Phylum notes is done simply by looking for notes that start with a known string value.
         #       We only care about the most recent Phylum note.
         for mr_note in mr_notes:
             if mr_note.get("body", "").lstrip().startswith(PHYLUM_HEADER.strip()):
