@@ -3,6 +3,7 @@ import argparse
 import json
 import os
 import pathlib
+import shlex
 import subprocess
 import sys
 from typing import List, Optional, Sequence, Tuple
@@ -61,12 +62,53 @@ def detect_ci_platform(args: argparse.Namespace, remainder: List[str]) -> CIBase
     return ci_env
 
 
+def ensure_project(ci_env: CIBase) -> None:
+    """Ensure a Phylum project is created and in place.
+
+    When a project is specified through arguments, attempt to create that project,
+    overwriting any `.phylum_project` file that already exists.
+    Continue on without error when a specified project already exists.
+    When a group is specified through arguments, attempt to use that group.
+    """
+    if not ci_env.phylum_project:
+        return
+
+    cmd = f"{ci_env.cli_path} project create {ci_env.phylum_project}"
+    if ci_env.phylum_group:
+        print(f" [-] Using Phylum group: {ci_env.phylum_group}")
+        cmd = f"{ci_env.cli_path} project create --group {ci_env.phylum_group} {ci_env.phylum_project}"
+
+    print(f" [*] Creating a Phylum project with the name: {ci_env.phylum_project} ...")
+    if ci_env.phylum_project_file.exists():
+        print(f" [+] Overwriting existing `.phylum_project` file found at: {ci_env.phylum_project_file}")
+
+    ret = subprocess.run(shlex.split(cmd), check=False)
+    # The Phylum CLI will return a unique error code of 14 when a project that already
+    # exists is attempted to be created. This situation is recognized and allowed to happen
+    # since it means the project exists as expected. Any other exit code is an error.
+    if ret.returncode == 0:
+        print(f" [-] Project {ci_env.phylum_project} created successfully.")
+    elif ret.returncode == 14:
+        print(f" [-] Project {ci_env.phylum_project} already exists. Continuing with it ...")
+    else:
+        print(f" [!] There was a problem creating the project with command: {cmd}")
+        ret.check_returncode()
+
+
 def get_phylum_analysis(ci_env: CIBase) -> dict:
     """Analyze a project lockfile from a given CI environment with the phylum CLI and return the analysis."""
-    print(" [*] Performing analysis ...")
-    cmd = f"{ci_env.cli_path} analyze -l {ci_env.phylum_label} --verbose --json {ci_env.lockfile}".split()
+    # Build up the analyze command based on the provided inputs
+    cmd = f"{ci_env.cli_path} analyze -l {ci_env.phylum_label}"
+    if ci_env.phylum_project:
+        cmd = f"{cmd} --project {ci_env.phylum_project}"
+        # A group can not be specified without a project
+        if ci_env.phylum_group:
+            cmd = f"{cmd} --group {ci_env.phylum_group}"
+    cmd = f"{cmd} --verbose --json {ci_env.lockfile}"
+
+    print(f" [*] Performing analysis with command: {cmd} ...")
     try:
-        analysis_result = subprocess.run(cmd, check=True, capture_output=True, text=True).stdout
+        analysis_result = subprocess.run(shlex.split(cmd), check=True, capture_output=True, text=True).stdout
     except subprocess.CalledProcessError as err:
         # The Phylum project can set the CLI to "fail the build" if threshold requirements are not met.
         # This causes the return code to be non-zero and lands us here. Check for this case to proceed.
@@ -138,6 +180,20 @@ def get_args(args: Optional[Sequence[str]] = None) -> Tuple[argparse.Namespace,
             already set in the Phylum config file or (2) to manually populate the token with a `phylum auth login` or
             `phylum auth register` command after install.""",
     )
+    analysis_group.add_argument(
+        "-p",
+        "--project",
+        # NOTE: The method of using the shlex module is known to be compatible with UNIX shells.
+        #       It may not function as desired for other operating systems and/or shell types.
+        type=shlex.quote,
+        help="Name of a Phylum project to create and use to perform the analysis.",
+    )
+    analysis_group.add_argument(
+        "-g",
+        "--group",
+        type=shlex.quote,
+        help="Optional group name, which will be the owner of the project. Only used when a project is also specified.",
+    )
 
     threshold_group = parser.add_argument_group(
         title="Risk Domain Threshold Options",
@@ -235,6 +291,9 @@ def main(args: Optional[Sequence[str]] = None) -> int:
     # Check for the existence of the CLI and install it if needed
     ci_env.init_cli()
 
+    # Ensure a Phylum project is created and in place
+    ensure_project(ci_env)
+
     # Analyze current project lockfile with phylum CLI
     analysis = get_phylum_analysis(ci_env)
 
